@@ -28,8 +28,6 @@
  * This is the base class for audio sinks. Subclasses need to implement the
  * ::create_ringbuffer vmethod. This base class will then take care of
  * writing samples to the ringbuffer, synchronisation, clipping and flushing.
- *
- * Last reviewed on 2006-09-27 (0.10.12)
  */
 
 #include <string.h>
@@ -80,7 +78,7 @@ enum
   LAST_SIGNAL
 };
 
-/* FIXME: 0.11, store the buffer_time and latency_time in nanoseconds */
+/* FIXME: 2.0, store the buffer_time and latency_time in nanoseconds */
 #define DEFAULT_BUFFER_TIME     ((200 * GST_MSECOND) / GST_USECOND)
 #define DEFAULT_LATENCY_TIME    ((10 * GST_MSECOND) / GST_USECOND)
 #define DEFAULT_PROVIDE_CLOCK   TRUE
@@ -90,7 +88,7 @@ enum
 #define DEFAULT_CAN_ACTIVATE_PULL FALSE
 
 /* when timestamps drift for more than 40ms we resync. This should
- * be anough to compensate for timestamp rounding errors. */
+ * be enough to compensate for timestamp rounding errors. */
 #define DEFAULT_ALIGNMENT_THRESHOLD   (40 * GST_MSECOND)
 
 /* when clock slaving drift for more than 40ms we resync. This is
@@ -152,10 +150,6 @@ static void gst_audio_base_sink_set_property (GObject * object, guint prop_id,
 static void gst_audio_base_sink_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 
-#if 0
-static GstStateChangeReturn gst_audio_base_sink_async_play (GstBaseSink *
-    basesink);
-#endif
 static GstStateChangeReturn gst_audio_base_sink_change_state (GstElement *
     element, GstStateChange transition);
 static gboolean gst_audio_base_sink_activate_pull (GstBaseSink * basesink,
@@ -164,6 +158,7 @@ static gboolean gst_audio_base_sink_query (GstElement * element, GstQuery *
     query);
 
 static GstClock *gst_audio_base_sink_provide_clock (GstElement * elem);
+static inline void gst_audio_base_sink_reset_sync (GstAudioBaseSink * sink);
 static GstClockTime gst_audio_base_sink_get_time (GstClock * clock,
     GstAudioBaseSink * sink);
 static void gst_audio_base_sink_callback (GstAudioRingBuffer * rbuf,
@@ -215,8 +210,8 @@ gst_audio_base_sink_class_init (GstAudioBaseSinkClass * klass)
 
   g_object_class_install_property (gobject_class, PROP_LATENCY_TIME,
       g_param_spec_int64 ("latency-time", "Latency Time",
-          "The minimum amount of data to write in each iteration in microseconds",
-          1, G_MAXINT64, DEFAULT_LATENCY_TIME,
+          "The minimum amount of data to write in each iteration "
+          "in microseconds", 1, G_MAXINT64, DEFAULT_LATENCY_TIME,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class, PROP_PROVIDE_CLOCK,
@@ -226,7 +221,7 @@ gst_audio_base_sink_class_init (GstAudioBaseSinkClass * klass)
 
   g_object_class_install_property (gobject_class, PROP_SLAVE_METHOD,
       g_param_spec_enum ("slave-method", "Slave Method",
-          "Algorithm to use to match the rate of the masterclock",
+          "Algorithm used to match the rate of the masterclock",
           GST_TYPE_AUDIO_BASE_SINK_SLAVE_METHOD, DEFAULT_SLAVE_METHOD,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
@@ -450,7 +445,7 @@ gst_audio_base_sink_query (GstElement * element, GstQuery * query)
             GST_OBJECT_UNLOCK (basesink);
 
             GST_DEBUG_OBJECT (basesink,
-                "we are not yet negotiated, can't report latency yet");
+                "we are not negotiated, can't report latency yet");
             res = FALSE;
             goto done;
           }
@@ -469,6 +464,7 @@ gst_audio_base_sink_query (GstElement * element, GstQuery * query)
            * amount of time. */
           max_latency = (max_l == -1) ? -1 : (base_latency + max_l);
 
+
           GST_DEBUG_OBJECT (basesink,
               "peer min %" GST_TIME_FORMAT ", our min latency: %"
               GST_TIME_FORMAT, GST_TIME_ARGS (min_l),
@@ -561,7 +557,8 @@ gst_audio_base_sink_get_time (GstClock * clock, GstAudioBaseSink * sink)
  *
  * Controls whether @sink will provide a clock or not. If @provide is %TRUE,
  * gst_element_provide_clock() will return a clock that reflects the datarate
- * of @sink. If @provide is %FALSE, gst_element_provide_clock() will return NULL.
+ * of @sink. If @provide is %FALSE, gst_element_provide_clock() will return
+ * NULL.
  */
 void
 gst_audio_base_sink_set_provide_clock (GstAudioBaseSink * sink,
@@ -891,10 +888,7 @@ gst_audio_base_sink_setcaps (GstBaseSink * bsink, GstCaps * caps)
     goto acquire_error;
 
   /* We need to resync since the ringbuffer restarted */
-  sink->priv->avg_skew = -1;
-  sink->next_sample = -1;
-  sink->priv->eos_time = -1;
-  sink->priv->discont_time = -1;
+  gst_audio_base_sink_reset_sync (sink);
 
   if (bsink->pad_mode == GST_PAD_MODE_PUSH) {
     GST_DEBUG_OBJECT (sink, "activate ringbuffer");
@@ -909,7 +903,7 @@ gst_audio_base_sink_setcaps (GstBaseSink * bsink, GstCaps * caps)
       crate_denom);
 
   /* calculate actual latency and buffer times.
-   * FIXME: In 0.11, store the latency_time internally in ns */
+   * FIXME: In 2.0, store the latency_time internally in ns */
   spec->latency_time = gst_util_uint64_scale (spec->segsize,
       (GST_SECOND / GST_USECOND), spec->info.rate * spec->info.bpf);
 
@@ -966,6 +960,16 @@ gst_audio_base_sink_fixate (GstBaseSink * bsink, GstCaps * caps)
   return caps;
 }
 
+static inline void
+gst_audio_base_sink_reset_sync (GstAudioBaseSink * sink)
+{
+  sink->next_sample = -1;
+  sink->priv->eos_time = -1;
+  sink->priv->discont_time = -1;
+  sink->priv->avg_skew = -1;
+  sink->priv->last_align = 0;
+}
+
 static void
 gst_audio_base_sink_get_times (GstBaseSink * bsink, GstBuffer * buffer,
     GstClockTime * start, GstClockTime * end)
@@ -1086,10 +1090,7 @@ gst_audio_base_sink_event (GstBaseSink * bsink, GstEvent * event)
       break;
     case GST_EVENT_FLUSH_STOP:
       /* always resync on sample after a flush */
-      sink->priv->avg_skew = -1;
-      sink->next_sample = -1;
-      sink->priv->eos_time = -1;
-      sink->priv->discont_time = -1;
+      gst_audio_base_sink_reset_sync (sink);
       if (sink->ringbuffer)
         gst_audio_ring_buffer_set_flushing (sink->ringbuffer, FALSE);
       break;
@@ -1123,8 +1124,8 @@ wrong_state:
 static guint64
 gst_audio_base_sink_get_offset (GstAudioBaseSink * sink)
 {
-  guint64 sample;
-  gint writeseg, segdone, sps;
+  guint64 sample, sps;
+  gint writeseg, segdone;
   gint diff;
 
   /* assume we can append to the previous sample */
@@ -1285,14 +1286,20 @@ gst_audio_base_sink_skew_slaving (GstAudioBaseSink * sink,
     GST_WARNING_OBJECT (sink,
         "correct clock skew %" G_GINT64_FORMAT " > %" G_GINT64_FORMAT,
         sink->priv->avg_skew, mdrift2);
-    cexternal = cexternal > mdrift ? cexternal - mdrift : 0;
-    sink->priv->avg_skew -= mdrift;
+
+    if (sink->priv->avg_skew > (2 * mdrift)) {
+      cexternal -= sink->priv->avg_skew;
+      sink->priv->avg_skew = 0;
+    } else {
+      cexternal = cexternal > mdrift ? cexternal - mdrift : 0;
+      sink->priv->avg_skew -= mdrift;
+    }
 
     driftsamples = (sink->ringbuffer->spec.info.rate * mdrift) / GST_SECOND;
     last_align = sink->priv->last_align;
 
-    /* if we were aligning in the wrong direction or we aligned more than what we
-     * will correct, resync */
+    /* if we were aligning in the wrong direction or we aligned more than what
+     * we will correct, resync */
     if (last_align < 0 || last_align > driftsamples)
       sink->next_sample = -1;
 
@@ -1307,14 +1314,20 @@ gst_audio_base_sink_skew_slaving (GstAudioBaseSink * sink,
     GST_WARNING_OBJECT (sink,
         "correct clock skew %" G_GINT64_FORMAT " < %" G_GINT64_FORMAT,
         sink->priv->avg_skew, -mdrift2);
-    cexternal += mdrift;
-    sink->priv->avg_skew += mdrift;
+
+    if (sink->priv->avg_skew < (2 * -mdrift)) {
+      cexternal -= sink->priv->avg_skew;
+      sink->priv->avg_skew = 0;
+    } else {
+      cexternal += mdrift;
+      sink->priv->avg_skew += mdrift;
+    }
 
     driftsamples = (sink->ringbuffer->spec.info.rate * mdrift) / GST_SECOND;
     last_align = sink->priv->last_align;
 
-    /* if we were aligning in the wrong direction or we aligned more than what we
-     * will correct, resync */
+    /* if we were aligning in the wrong direction or we aligned more than what
+     * we will correct, resync */
     if (last_align > 0 || -last_align > driftsamples)
       sink->next_sample = -1;
 
@@ -1451,15 +1464,21 @@ gst_audio_base_sink_sync_latency (GstBaseSink * bsink, GstMiniObject * obj)
      * yet. if some other error occures, we continue. */
   } while (status == GST_CLOCK_UNSCHEDULED);
 
-  GST_OBJECT_LOCK (sink);
   GST_DEBUG_OBJECT (sink, "latency synced");
 
+  /* Do not acquire the sink object lock before trying to get time on the Sink. 
+   * The get_time call may need to acquire the pulse audio mainloop lock. This can 
+   * cause a deadlock with the Pulse Audio main loop thread which in turn has 
+   * acquired its mainloop lock and now needs to acquire Object lock on the sink. */
+  itime = gst_audio_clock_get_time (sink->provided_clock);
+  itime = gst_audio_clock_adjust (sink->provided_clock, itime);
+
+  GST_OBJECT_LOCK (sink);
+
   /* when we prerolled in time, we can accurately set the calibration,
    * our internal clock should exactly have been the latency (== the running
    * time of the external clock) */
   etime = GST_ELEMENT_CAST (sink)->base_time + time;
-  itime = gst_audio_clock_get_time (sink->provided_clock);
-  itime = gst_audio_clock_adjust (sink->provided_clock, itime);
 
   if (status == GST_CLOCK_EARLY) {
     /* when we prerolled late, we have to take into account the lateness */
@@ -1493,10 +1512,7 @@ gst_audio_base_sink_sync_latency (GstBaseSink * bsink, GstMiniObject * obj)
       break;
   }
 
-  sink->priv->avg_skew = -1;
-  sink->next_sample = -1;
-  sink->priv->eos_time = -1;
-  sink->priv->discont_time = -1;
+  gst_audio_base_sink_reset_sync (sink);
 
   return GST_FLOW_OK;
 
@@ -1528,28 +1544,24 @@ gst_audio_base_sink_get_alignment (GstAudioBaseSink * sink,
   gint64 sample_diff;
   gint64 max_sample_diff;
   gint segdone = g_atomic_int_get (&ringbuf->segdone) - ringbuf->segbase;
-  gint64 samples_done = segdone * ringbuf->samples_per_seg;
+  gint64 samples_done = segdone * (gint64) ringbuf->samples_per_seg;
   gint64 headroom = sample_offset - samples_done;
   gboolean allow_align = TRUE;
   gboolean discont = FALSE;
   gint rate;
 
-  /* now try to align the sample to the previous one, first see how big the
-   * difference is. */
-  if (sample_offset >= sink->next_sample)
-    sample_diff = sample_offset - sink->next_sample;
-  else
-    sample_diff = sink->next_sample - sample_offset;
+  /* now try to align the sample to the previous one. */
 
-  rate = GST_AUDIO_INFO_RATE (&ringbuf->spec.info);
+  /* calc align with previous sample and determine how big the
+   * difference is. */
+  align = sink->next_sample - sample_offset;
+  sample_diff = ABS (align);
 
   /* calculate the max allowed drift in units of samples. */
+  rate = GST_AUDIO_INFO_RATE (&ringbuf->spec.info);
   max_sample_diff = gst_util_uint64_scale_int (sink->priv->alignment_threshold,
       rate, GST_SECOND);
 
-  /* calc align with previous sample */
-  align = sink->next_sample - sample_offset;
-
   /* don't align if it means writing behind the read-segment */
   if (sample_diff > headroom && align < 0)
     allow_align = FALSE;
@@ -1785,8 +1797,8 @@ gst_audio_base_sink_render (GstBaseSink * bsink, GstBuffer * buf)
   }
 
   if (G_UNLIKELY (sync_offset != 0)) {
-    /* compensate for ts-offset and delay we know this will not underflow because we
-     * clipped above. */
+    /* compensate for ts-offset and delay. We know this will not underflow
+     * because we clipped above. */
     GST_DEBUG_OBJECT (sink,
         "compensating for sync-offset %" GST_TIME_FORMAT,
         GST_TIME_ARGS (sync_offset));
@@ -1876,8 +1888,9 @@ gst_audio_base_sink_render (GstBaseSink * bsink, GstBuffer * buf)
   }
 
   /* always resync after a discont */
-  if (G_UNLIKELY (GST_BUFFER_FLAG_IS_SET (buf, GST_BUFFER_FLAG_DISCONT))) {
-    GST_DEBUG_OBJECT (sink, "resync after discont");
+  if (G_UNLIKELY (GST_BUFFER_FLAG_IS_SET (buf, GST_BUFFER_FLAG_DISCONT) ||
+          GST_BUFFER_FLAG_IS_SET (buf, GST_BUFFER_FLAG_RESYNC))) {
+    GST_DEBUG_OBJECT (sink, "resync after discont/resync");
     goto no_align;
   }
 
@@ -2029,9 +2042,9 @@ sync_latency_failed:
  * gst_audio_base_sink_create_ringbuffer:
  * @sink: a #GstAudioBaseSink.
  *
- * Create and return the #GstAudioRingBuffer for @sink. This function will call the
- * ::create_ringbuffer vmethod and will set @sink as the parent of the returned
- * buffer (see gst_object_set_parent()).
+ * Create and return the #GstAudioRingBuffer for @sink. This function will
+ * call the ::create_ringbuffer vmethod and will set @sink as the parent of
+ * the returned buffer (see gst_object_set_parent()).
  *
  * Returns: (transfer none): The new ringbuffer of @sink.
  */
@@ -2067,7 +2080,7 @@ gst_audio_base_sink_callback (GstAudioRingBuffer * rbuf, guint8 * data,
   GST_PAD_STREAM_LOCK (basesink->sinkpad);
 
   /* would be nice to arrange for pad_alloc_buffer to return data -- as it is we
-     will copy twice, once into data, once into DMA */
+   * will copy twice, once into data, once into DMA */
   GST_LOG_OBJECT (basesink, "pulling %u bytes offset %" G_GUINT64_FORMAT
       " to fill audio buffer", len, basesink->offset);
   ret =
@@ -2168,27 +2181,6 @@ gst_audio_base_sink_activate_pull (GstBaseSink * basesink, gboolean active)
   return ret;
 }
 
-#if 0
-/* should be called with the LOCK */
-static GstStateChangeReturn
-gst_audio_base_sink_async_play (GstBaseSink * basesink)
-{
-  GstAudioBaseSink *sink;
-
-  sink = GST_AUDIO_BASE_SINK (basesink);
-
-  GST_DEBUG_OBJECT (sink, "ringbuffer may start now");
-  sink->priv->sync_latency = TRUE;
-  gst_audio_ring_buffer_may_start (sink->ringbuffer, TRUE);
-  if (basesink->pad_mode == GST_PAD_MODE_PULL) {
-    /* we always start the ringbuffer in pull mode immediatly */
-    gst_audio_ring_buffer_start (sink->ringbuffer);
-  }
-
-  return GST_STATE_CHANGE_SUCCESS;
-}
-#endif
-
 static GstStateChangeReturn
 gst_audio_base_sink_change_state (GstElement * element,
     GstStateChange transition)
@@ -2206,10 +2198,7 @@ gst_audio_base_sink_change_state (GstElement * element,
         goto open_failed;
       break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
-      sink->next_sample = -1;
-      sink->priv->last_align = -1;
-      sink->priv->eos_time = -1;
-      sink->priv->discont_time = -1;
+      gst_audio_base_sink_reset_sync (sink);
       gst_audio_ring_buffer_set_flushing (sink->ringbuffer, FALSE);
       gst_audio_ring_buffer_may_start (sink->ringbuffer, FALSE);
 
@@ -2283,7 +2272,7 @@ gst_audio_base_sink_change_state (GstElement * element,
       gst_audio_ring_buffer_release (sink->ringbuffer);
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
-      /* we release again here because the aqcuire happens when setting the
+      /* we release again here because the acquire happens when setting the
        * caps, which happens before we commit the state to PAUSED and thus the
        * PAUSED->READY state change (see above, where we release the ringbuffer)
        * might not be called when we get here. */
