@@ -16,8 +16,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -47,7 +47,7 @@
  *   </varlistentry>
  *   <varlistentry>
  *     <term>delay()</term>
- *     <listitem><para>Get the number of samples written but not yet played 
+ *     <listitem><para>Get the number of samples written but not yet played
  *     by the device.</para></listitem>
  *   </varlistentry>
  *   <varlistentry>
@@ -63,16 +63,13 @@
  * All scheduling of samples and timestamps is done in this base class
  * together with #GstAudioBaseSink using a default implementation of a
  * #GstAudioRingBuffer that uses threads.
- *
- * Last reviewed on 2006-09-27 (0.10.12)
  */
 
 #include <string.h>
 
+#include <gst/audio/audio.h>
 #include "gstaudiosink.h"
 
-#include "gst/glib-compat-private.h"
-
 GST_DEBUG_CATEGORY_STATIC (gst_audio_sink_debug);
 #define GST_CAT_DEFAULT gst_audio_sink_debug
 
@@ -94,7 +91,7 @@ GST_DEBUG_CATEGORY_STATIC (gst_audio_sink_debug);
 typedef struct _GstAudioSinkRingBuffer GstAudioSinkRingBuffer;
 typedef struct _GstAudioSinkRingBufferClass GstAudioSinkRingBufferClass;
 
-#define GST_AUDIO_SINK_RING_BUFFER_GET_COND(buf) (((GstAudioSinkRingBuffer *)buf)->cond)
+#define GST_AUDIO_SINK_RING_BUFFER_GET_COND(buf) (&(((GstAudioSinkRingBuffer *)buf)->cond))
 #define GST_AUDIO_SINK_RING_BUFFER_WAIT(buf)     (g_cond_wait (GST_AUDIO_SINK_RING_BUFFER_GET_COND (buf), GST_OBJECT_GET_LOCK (buf)))
 #define GST_AUDIO_SINK_RING_BUFFER_SIGNAL(buf)   (g_cond_signal (GST_AUDIO_SINK_RING_BUFFER_GET_COND (buf)))
 #define GST_AUDIO_SINK_RING_BUFFER_BROADCAST(buf)(g_cond_broadcast (GST_AUDIO_SINK_RING_BUFFER_GET_COND (buf)))
@@ -106,7 +103,7 @@ struct _GstAudioSinkRingBuffer
   gboolean running;
   gint queuedseg;
 
-  GCond *cond;
+  GCond cond;
 };
 
 struct _GstAudioSinkRingBufferClass
@@ -205,7 +202,7 @@ typedef gint (*WriteFunc) (GstAudioSink * sink, gpointer data, guint length);
 
 /* this internal thread does nothing else but write samples to the audio device.
  * It will write each segment in the ringbuffer and will update the play
- * pointer. 
+ * pointer.
  * The start/stop methods control the thread.
  */
 static void
@@ -232,11 +229,12 @@ audioringbuffer_thread_func (GstAudioRingBuffer * buf)
   if (writefunc == NULL)
     goto no_function;
 
-  g_value_init (&val, G_TYPE_POINTER);
-  g_value_set_pointer (&val, sink->thread);
   message = gst_message_new_stream_status (GST_OBJECT_CAST (buf),
       GST_STREAM_STATUS_TYPE_ENTER, GST_ELEMENT_CAST (sink));
+  g_value_init (&val, GST_TYPE_G_THREAD);
+  g_value_set_boxed (&val, sink->thread);
   gst_message_set_stream_status_object (message, &val);
+  g_value_unset (&val);
   GST_DEBUG_OBJECT (sink, "posting ENTER stream status");
   gst_element_post_message (GST_ELEMENT_CAST (sink), message);
 
@@ -308,7 +306,10 @@ stop_running:
     GST_DEBUG_OBJECT (sink, "stop running, exit thread");
     message = gst_message_new_stream_status (GST_OBJECT_CAST (buf),
         GST_STREAM_STATUS_TYPE_LEAVE, GST_ELEMENT_CAST (sink));
+    g_value_init (&val, GST_TYPE_G_THREAD);
+    g_value_set_boxed (&val, sink->thread);
     gst_message_set_stream_status_object (message, &val);
+    g_value_unset (&val);
     GST_DEBUG_OBJECT (sink, "posting LEAVE stream status");
     gst_element_post_message (GST_ELEMENT_CAST (sink), message);
     return;
@@ -322,7 +323,7 @@ gst_audio_sink_ring_buffer_init (GstAudioSinkRingBuffer * ringbuffer,
   ringbuffer->running = FALSE;
   ringbuffer->queuedseg = 0;
 
-  ringbuffer->cond = g_cond_new ();
+  g_cond_init (&ringbuffer->cond);
 }
 
 static void
@@ -336,7 +337,7 @@ gst_audio_sink_ring_buffer_finalize (GObject * object)
 {
   GstAudioSinkRingBuffer *ringbuffer = GST_AUDIO_SINK_RING_BUFFER_CAST (object);
 
-  g_cond_free (ringbuffer->cond);
+  g_cond_clear (&ringbuffer->cond);
 
   G_OBJECT_CLASS (ring_parent_class)->finalize (object);
 }
@@ -398,6 +399,7 @@ gst_audio_sink_ring_buffer_acquire (GstAudioRingBuffer * buf,
   GstAudioSink *sink;
   GstAudioSinkClass *csink;
   gboolean result = FALSE;
+  GstAudioClock *clock;
 
   sink = GST_AUDIO_SINK (GST_OBJECT_PARENT (buf));
   csink = GST_AUDIO_SINK_GET_CLASS (sink);
@@ -407,6 +409,10 @@ gst_audio_sink_ring_buffer_acquire (GstAudioRingBuffer * buf,
   if (!result)
     goto could_not_prepare;
 
+  /* our clock will now start from 0 again */
+  clock = GST_AUDIO_CLOCK (GST_AUDIO_BASE_SINK (sink)->provided_clock);
+  gst_audio_clock_reset (clock, 0);
+
   /* set latency to one more segment as we need some headroom */
   spec->seglatency = spec->segtotal + 1;
 
