@@ -46,8 +46,6 @@
  * @short_description: dealing with RTSP transports
  *  
  * Provides helper functions to deal with RTSP transport strings.
- *  
- * Last reviewed on 2007-07-25 (0.10.14)
  */
 
 #include <string.h>
@@ -77,15 +75,33 @@ typedef struct
 {
   const gchar *name;
   const GstRTSPTransMode mode;
-  const gchar *gst_mime;
+  const GstRTSPProfile profile;
+  const GstRTSPLowerTrans ltrans;
+  const gchar *media_type;
   const gchar *manager[MAX_MANAGERS];
 } GstRTSPTransMap;
 
 static const GstRTSPTransMap transports[] = {
-  {"rtp", GST_RTSP_TRANS_RTP, "application/x-rtp", {"rtpbin", "rtpdec"}},
-  {"x-real-rdt", GST_RTSP_TRANS_RDT, "application/x-rdt", {"rdtmanager", NULL}},
-  {"x-pn-tng", GST_RTSP_TRANS_RDT, "application/x-rdt", {"rdtmanager", NULL}},
-  {NULL, GST_RTSP_TRANS_UNKNOWN, NULL, {NULL, NULL}}
+  {"rtp", GST_RTSP_TRANS_RTP, GST_RTSP_PROFILE_AVP,
+        GST_RTSP_LOWER_TRANS_UDP_MCAST, "application/x-rtp",
+      {"rtpbin", "rtpdec"}},
+  {"srtp", GST_RTSP_TRANS_RTP, GST_RTSP_PROFILE_SAVP,
+        GST_RTSP_LOWER_TRANS_UDP_MCAST, "application/x-srtp",
+      {"rtpbin", "rtpdec"}},
+  {"rtpf", GST_RTSP_TRANS_RTP, GST_RTSP_PROFILE_AVPF,
+        GST_RTSP_LOWER_TRANS_UDP_MCAST, "application/x-rtp",
+      {"rtpbin", "rtpdec"}},
+  {"srtpf", GST_RTSP_TRANS_RTP, GST_RTSP_PROFILE_SAVPF,
+        GST_RTSP_LOWER_TRANS_UDP_MCAST, "application/x-srtp",
+      {"rtpbin", "rtpdec"}},
+  {"x-real-rdt", GST_RTSP_TRANS_RDT, GST_RTSP_PROFILE_AVP,
+        GST_RTSP_LOWER_TRANS_UNKNOWN, "application/x-rdt",
+      {"rdtmanager", NULL}},
+  {"x-pn-tng", GST_RTSP_TRANS_RDT, GST_RTSP_PROFILE_AVP,
+        GST_RTSP_LOWER_TRANS_UNKNOWN, "application/x-rdt",
+      {"rdtmanager", NULL}},
+  {NULL, GST_RTSP_TRANS_UNKNOWN, GST_RTSP_PROFILE_UNKNOWN,
+      GST_RTSP_LOWER_TRANS_UNKNOWN, NULL, {NULL, NULL}}
 };
 
 typedef struct
@@ -97,6 +113,8 @@ typedef struct
 static const RTSPProfileMap profiles[] = {
   {"avp", GST_RTSP_PROFILE_AVP},
   {"savp", GST_RTSP_PROFILE_SAVP},
+  {"avpf", GST_RTSP_PROFILE_AVPF},
+  {"savpf", GST_RTSP_PROFILE_SAVPF},
   {NULL, GST_RTSP_PROFILE_UNKNOWN}
 };
 
@@ -107,6 +125,8 @@ gst_rtsp_profile_get_type (void)
   static const GFlagsValue rtsp_profile[] = {
     {GST_RTSP_PROFILE_AVP, "GST_RTSP_PROFILE_AVP", "avp"},
     {GST_RTSP_PROFILE_SAVP, "GST_RTSP_PROFILE_SAVP", "savp"},
+    {GST_RTSP_PROFILE_AVPF, "GST_RTSP_PROFILE_AVPF", "avpf"},
+    {GST_RTSP_PROFILE_SAVPF, "GST_RTSP_PROFILE_SAVPF", "savpf"},
     {0, NULL, NULL},
   };
 
@@ -224,9 +244,13 @@ gst_rtsp_transport_init (GstRTSPTransport * transport)
  * @mime: location to hold the result
  *
  * Get the mime type of the transport mode @trans. This mime type is typically
- * used to generate #GstCaps on buffers.
+ * used to generate #GstCaps events.
  *
- * Returns: #GST_RTSP_OK. 
+ * Deprecated: This functions only deals with the GstRTSPTransMode and only
+ *    returns the mime type for #GST_RTSP_PROFILE_AVP. Use
+ *    gst_rtsp_transport_get_media_type() instead.
+ *
+ * Returns: #GST_RTSP_OK.
  */
 GstRTSPResult
 gst_rtsp_transport_get_mime (GstRTSPTransMode trans, const gchar ** mime)
@@ -236,13 +260,57 @@ gst_rtsp_transport_get_mime (GstRTSPTransMode trans, const gchar ** mime)
   g_return_val_if_fail (mime != NULL, GST_RTSP_EINVAL);
 
   for (i = 0; transports[i].name; i++)
-    if (transports[i].mode == trans)
+    if (transports[i].mode == trans
+        && transports[i].profile == GST_RTSP_PROFILE_AVP)
       break;
-  *mime = transports[i].gst_mime;
+  *mime = transports[i].media_type;
 
   return GST_RTSP_OK;
 }
 
+/**
+ * gst_rtsp_transport_get_media_type:
+ * @transport: a #GstRTSPTransport
+ * @media_type: (out) (transfer none): media type of @transport
+ *
+ * Get the media type of @transport. This media type is typically
+ * used to generate #GstCaps events.
+ *
+ * Since: 1.4
+ *
+ * Returns: #GST_RTSP_OK.
+ */
+GstRTSPResult
+gst_rtsp_transport_get_media_type (GstRTSPTransport * transport,
+    const gchar ** media_type)
+{
+  gint i;
+
+  g_return_val_if_fail (transport != NULL, GST_RTSP_EINVAL);
+  g_return_val_if_fail (media_type != NULL, GST_RTSP_EINVAL);
+
+  for (i = 0; transports[i].name; i++)
+    if (transports[i].mode == transport->trans
+        && transports[i].profile == transport->profile)
+      break;
+  *media_type = transports[i].media_type;
+
+  return GST_RTSP_OK;
+}
+
+static GstRTSPLowerTrans
+get_default_lower_trans (GstRTSPTransport * transport)
+{
+  gint i;
+
+  for (i = 0; transports[i].name; i++)
+    if (transports[i].mode == transport->trans
+        && transports[i].profile == transport->profile)
+      break;
+
+  return transports[i].ltrans;
+}
+
 /**
  * gst_rtsp_transport_get_manager:
  * @trans: a #GstRTSPTransMode
@@ -457,11 +525,7 @@ gst_rtsp_transport_parse (const gchar * str, GstRTSPTransport * transport)
     transport->lower_transport = ltrans[i].ltrans;
   } else {
     /* specifying the lower transport is optional */
-    if (transport->trans == GST_RTSP_TRANS_RTP &&
-        transport->profile == GST_RTSP_PROFILE_AVP)
-      transport->lower_transport = GST_RTSP_LOWER_TRANS_UDP_MCAST;
-    else
-      transport->lower_transport = GST_RTSP_LOWER_TRANS_UNKNOWN;
+    transport->lower_transport = get_default_lower_trans (transport);
   }
 
   g_strfreev (transp);
@@ -589,7 +653,10 @@ gst_rtsp_transport_as_text (GstRTSPTransport * transport)
   g_ptr_array_add (strs, g_ascii_strup (tmp, -1));
 
   if (transport->trans != GST_RTSP_TRANS_RTP ||
-      transport->profile != GST_RTSP_PROFILE_AVP ||
+      (transport->profile != GST_RTSP_PROFILE_AVP &&
+          transport->profile != GST_RTSP_PROFILE_SAVP &&
+          transport->profile != GST_RTSP_PROFILE_AVPF &&
+          transport->profile != GST_RTSP_PROFILE_SAVPF) ||
       transport->lower_transport == GST_RTSP_LOWER_TRANS_TCP) {
     g_ptr_array_add (strs, g_strdup ("/"));
 
