@@ -77,13 +77,13 @@
 #define EXPORT G_GNUC_INTERNAL
 
 #ifdef _USE_SSE
-#ifndef HAVE_XMMINTRIN_H
+#if !defined(__SSE__) || !defined(HAVE_XMMINTRIN_H)
 #undef _USE_SSE
 #endif
 #endif
 
 #ifdef _USE_SSE2
-#ifndef HAVE_EMMINTRIN_H
+#if !defined(__SSE2__) || !defined(HAVE_EMMINTRIN_H)
 #undef _USE_SSE2
 #endif
 #endif
@@ -478,7 +478,7 @@ resampler_basic_direct_single (SpeexResamplerState * st,
     const spx_word16_t *iptr = &in[last_sample];
 
     SSE_FALLBACK (INNER_PRODUCT_SINGLE)
-    NEON_FALLBACK (INNER_PRODUCT_SINGLE)
+        NEON_FALLBACK (INNER_PRODUCT_SINGLE)
         sum = 0;
     for (j = 0; j < N; j++)
       sum += MULT16_16 (sinc[j], iptr[j]);
@@ -497,14 +497,14 @@ resampler_basic_direct_single (SpeexResamplerState * st,
 */
 #if defined(OVERRIDE_INNER_PRODUCT_SINGLE) && defined(_USE_NEON)
     NEON_IMPLEMENTATION (INNER_PRODUCT_SINGLE)
-    sum = inner_product_single (sinc, iptr, N);
-    NEON_END(INNER_PRODUCT_SINGLE)
+        sum = inner_product_single (sinc, iptr, N);
+    NEON_END (INNER_PRODUCT_SINGLE)
 #elif defined(OVERRIDE_INNER_PRODUCT_SINGLE) && defined(_USE_SSE)
     SSE_IMPLEMENTATION (INNER_PRODUCT_SINGLE)
         sum = inner_product_single (sinc, iptr, N);
     SSE_END (INNER_PRODUCT_SINGLE)
 #endif
-    out[out_stride * out_sample++] = SATURATE32PSHR(sum, 15, 32767);
+        out[out_stride * out_sample++] = SATURATE32PSHR (sum, 15, 32767);
     last_sample += int_advance;
     samp_frac_num += frac_advance;
     if (samp_frac_num >= den_rate) {
@@ -642,7 +642,7 @@ resampler_basic_interpolate_single (SpeexResamplerState * st,
         interp);
     SSE_END (INTERPOLATE_PRODUCT_SINGLE)
 #endif
-    out[out_stride * out_sample++] = SATURATE32PSHR(sum, 14, 32767);
+        out[out_stride * out_sample++] = SATURATE32PSHR (sum, 14, 32767);
     last_sample += int_advance;
     samp_frac_num += frac_advance;
     if (samp_frac_num >= den_rate) {
@@ -946,7 +946,8 @@ update_filter (SpeexResamplerState * st)
 
 EXPORT SpeexResamplerState *
 speex_resampler_init (spx_uint32_t nb_channels, spx_uint32_t in_rate,
-    spx_uint32_t out_rate, int quality, SpeexResamplerSincFilterMode sinc_filter_mode,
+    spx_uint32_t out_rate, int quality,
+    SpeexResamplerSincFilterMode sinc_filter_mode,
     spx_uint32_t sinc_filter_auto_threshold, int *err)
 {
   return speex_resampler_init_frac (nb_channels, in_rate, out_rate, in_rate,
@@ -988,6 +989,11 @@ speex_resampler_init_frac (spx_uint32_t nb_channels, spx_uint32_t ratio_num,
       *err = RESAMPLER_ERR_INVALID_ARG;
     return NULL;
   }
+  if (ratio_den == 0) {
+    if (err)
+      *err = RESAMPLER_ERR_INVALID_ARG;
+    return NULL;
+  }
   switch (sinc_filter_mode) {
     case RESAMPLER_SINC_FILTER_INTERPOLATED:
       use_full_sinc_table = 0;
@@ -1040,7 +1046,7 @@ speex_resampler_init_frac (spx_uint32_t nb_channels, spx_uint32_t ratio_num,
       unsigned int flags = orc_target_get_default_flags (target);
       check_insn_set (st, orc_target_get_name (target));
       for (i = 0; i < 32; ++i) {
-        if (flags & (1 << i)) {
+        if (flags & (1U << i)) {
           check_insn_set (st, orc_target_get_flag_name (target, i));
         }
       }
@@ -1063,13 +1069,16 @@ speex_resampler_init_frac (spx_uint32_t nb_channels, spx_uint32_t ratio_num,
 
   if (sinc_filter_mode == RESAMPLER_SINC_FILTER_AUTO) {
     /*
-    Estimate how big the filter table would become if the full mode were to be used
-    calculations used correspond to the ones in update_filter()
-    if the size is bigger than the threshold, use interpolated sinc instead
-    */
-    spx_uint32_t base_filter_length = st->filt_len = quality_map[st->quality].base_length;
-    spx_uint32_t filter_table_size = base_filter_length * st->den_rate * sizeof(spx_uint16_t);
-    st->use_full_sinc_table = (filter_table_size > sinc_filter_auto_threshold) ? 0 : 1;
+       Estimate how big the filter table would become if the full mode were to be used
+       calculations used correspond to the ones in update_filter()
+       if the size is bigger than the threshold, use interpolated sinc instead
+     */
+    spx_uint32_t base_filter_length = st->filt_len =
+        quality_map[st->quality].base_length;
+    spx_uint32_t filter_table_size =
+        base_filter_length * st->den_rate * sizeof (spx_uint16_t);
+    st->use_full_sinc_table =
+        (filter_table_size > sinc_filter_auto_threshold) ? 0 : 1;
   }
 
   update_filter (st);
@@ -1174,7 +1183,7 @@ speex_resampler_process_float (SpeexResamplerState * st,
   if (st->magic_samples[channel_index])
     olen -= speex_resampler_magic (st, channel_index, &out, olen);
   if (!st->magic_samples[channel_index]) {
-    while (ilen && olen) {
+    while (ilen) {
       spx_uint32_t ichunk = (ilen > xlen) ? xlen : ilen;
       spx_uint32_t ochunk = olen;
 
@@ -1229,7 +1238,7 @@ speex_resampler_process_int (SpeexResamplerState * st,
 
   st->out_stride = 1;
 
-  while (ilen && olen) {
+  while (ilen) {
     spx_word16_t *y = ystack;
     spx_uint32_t ichunk = (ilen > xlen) ? xlen : ilen;
     spx_uint32_t ochunk = (olen > ylen) ? ylen : olen;
